

#ifndef __WANPIPE_KERNEL_H
# define __WANPIPE_KERNEL_H


#if defined(WAN_KERNEL)

#if defined(__FreeBSD__)
/************************* F R E E B S D *************************************/
# define WAN_IFP2AC(ifp)			((struct arpcom*)ifp)
# define IFATTACH(ifp)			if_attach(ifp)
# define IFDETACH(ifp)			if_detach(ifp)
# if (__FreeBSD_version < 410000)
#  define ETHER_IFATTACH(ifp,enaddr)	if_attach(ifp); ether_ifattach(ifp)
#  define ETHER_IFDETACH(ifp)		if_detach(ifp)
# elif (__FreeBSD_version > 500000)
#  define ETHER_IFATTACH(ifp,enaddr)	ether_ifattach(ifp, enaddr)
#  define ETHER_IFDETACH(ifp)		ether_ifdetach(ifp)
# else
#  define ETHER_IFATTACH(ifp,enaddr)	ether_ifattach(ifp, ETHER_BPF_SUPPORTED)
#  define ETHER_IFDETACH(ifp)		ether_ifdetach(ifp, ETHER_BPF_SUPPORTED)
# endif
# define ETHER_IOCTL(ifp, cmd, data)	ether_ioctl(ifp, cmd, data)

# define WAN_MOD_LOAD			MOD_LOAD
# define WAN_MOD_UNLOAD			MOD_UNLOAD
# define wan_clear_bit(bit, name)	clear_bit(bit, name)
# define wan_test_bit(bit, name)	test_bit(bit, name)
# define wan_set_bit(bit, name)		set_bit(bit, name)
# define wan_test_and_set_bit(bit, name)	\
		(test_bit((bit), (name)) || set_bit((bit),(name)))
# define atomic_set(name, val)		*(int*)name = val
# define atomic_read(name)		*(int*)name
# define atomic_inc(name)		atomic_add_int(name, 1)
# define atomic_dec(name)		atomic_subtract_int(name, 1)
# define irqreturn_t			void
# define WAN_IRQ_RETVAL(a)		return;

# if (__FreeBSD_version > 500000)
typedef struct thread*			wan_proc_ptr_t;
# else
typedef struct proc*			wan_proc_ptr_t;
# endif
typedef struct arpcom	wan_ethercom_t;
#elif defined(__OpenBSD__)
/************************* O P E N B S D *************************************/
# define WAN_IFP2AC(ifp)		((struct arpcom*)ifp)
# define ETHER_IFATTACH(ifp, enaddr)	if_attach(ifp); ether_ifattach(ifp)
# define ETHER_IFDETACH(ifp)		ether_ifdetach(ifp); if_detach(ifp)
# define ETHER_IOCTL(ifp, cmd, data)	ether_ioctl(ifp, WAN_IFP2AC(ifp), cmd, data)
# define IFATTACH(ifp)			if_attach(ifp);if_alloc_sadl(ifp)
# define IFDETACH(ifp)			if_free_sadl(ifp); if_detach(ifp)

# define WAN_MOD_LOAD			LKM_E_LOAD
# define WAN_MOD_UNLOAD			LKM_E_UNLOAD
# define wan_clear_bit(bit, name)	bit_clear((unsigned char*)name, bit)
# define wan_set_bit(bit, name)		bit_set((unsigned char*)name, bit)
# define wan_test_bit(bit, name)	bit_test((unsigned char*)name, bit)
# define wan_test_and_set_bit(bit, name)		\
    	(bit_test((unsigned char*)(name),(bit)) || 	\
		!bit_set((unsigned char*)(name),(bit)))
# define atomic_set(name, val)		*(int*)name = val
# define atomic_set_int(name, val)	*(unsigned int*)name = val
# define atomic_read(name)		*(int*)name
# define atomic_inc(name)		*(int*)name += 1
# define atomic_dec(name)		*(int*)name -= 1 
# define irqreturn_t	void
/* Unsafe sprintf and vsprintf function removed from the kernel */
# define sprintf(a,b...)		snprintf(a, sizeof(a), ##b)
# define vsprintf(a,b...)		vsnprintf(a, sizeof(a), ##b)
# define WAN_IRQ_RETVAL(a)		return;

typedef struct proc*	wan_proc_ptr_t;
typedef struct arpcom	wan_ethercom_t;
#elif defined(__NetBSD__)
/************************* N E T B S D *************************************/
# define WAN_IFP2AC(ifp)		((struct ethercom*)ifp)
# define ETHER_IFATTACH(ifp,enaddr)	if_attach(ifp);ether_ifattach(ifp,enaddr)
# define ETHER_IFDETACH(ifp)		ether_ifdetach(ifp); if_detach(ifp)
# define ETHER_IOCTL(ifp, cmd, data)	ether_ioctl(ifp, cmd, data)
# define IFATTACH(ifp)			if_attach(ifp);if_alloc_sadl(ifp)
# define IFDETACH(ifp)			if_free_sadl(ifp); if_detach(ifp)

# define WAN_MOD_LOAD			LKM_E_LOAD
# define WAN_MOD_UNLOAD			LKM_E_UNLOAD
# define wan_clear_bit(bit, name)	bit_clear((unsigned char*)name, bit)
# define wan_set_bit(bit, name)		bit_set((unsigned char*)name, bit)
# define wan_test_bit(bit, name)	bit_test((unsigned char*)name, bit)
# define wan_test_and_set_bit(bit, name)		\
    	(bit_test((unsigned char*)(name),(bit)) || 	\
		!bit_set((unsigned char*)(name),(bit)))
# define atomic_set(name, val)		*(int*)name = val
# define atomic_set_int(name, val)	*(unsigned int*)name = val
# define atomic_read(name)		*(int*)name
# define atomic_inc(name)		*(int*)name += 1
# define atomic_dec(name)		*(int*)name -= 1 
# define irqreturn_t	void
/* Unsafe sprintf and vsprintf function removed from the kernel */
# define WAN_IRQ_RETVAL(a)		return;

typedef struct proc*	wan_proc_ptr_t;
typedef struct ethercom	wan_ethercom_t;
#else
# error "IFATTACH/IFDETACH/ETHER_IFDETACH macros doesn't supported yet!"
#endif


#endif /* WAN_KERNEL */

#endif /* __WANPIPE_KERNEL_H */
